﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace howto_overlay_image_parts
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private bool Drawing = false;
        private List<PointF> SelectedArea = new List<PointF>();
        private List<PointF> ScaledSelectedArea = new List<PointF>();

        private Bitmap OverlayImage = null;
        private Bitmap ScaledOverlayImage = null;

        private Bitmap OriginalMainImage = null;
        private Bitmap MainImage = null;
        private Bitmap ScaledMainImage = null;

        private float ImageScale = 1;

        // Let the user select an image file.
        private void mnuFileOpenOverlay_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    OverlayImage = LoadBitmapUnlocked(ofdImage.FileName);
                    mnuFileReset.Enabled = false;
                    ShowScaledImages();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileOpenMainImage_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    OriginalMainImage = LoadBitmapUnlocked(ofdImage.FileName);
                    MainImage = new Bitmap(OriginalMainImage);
                    mnuFileReset.Enabled = false;
                    ShowScaledImages();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        // Save the selected area.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                // Save the new Bitmap.
                SaveImage(MainImage, sfdImage.FileName);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }


        // Draw the selection area.
        private void picOverlay_Paint(object sender, PaintEventArgs e)
        {
            if (SelectedArea.Count < 3) return;

            // Draw the selection area.
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            PointF[] points = ScaledSelectedArea.ToArray();
            using (Pen pen = new Pen(Color.Red, 2))
            {
                if (Drawing) e.Graphics.DrawLines(pen, points);
                else e.Graphics.DrawPolygon(pen, points);

                pen.Color = Color.Yellow;
                pen.DashPattern = new float[] { 5, 5 };

                if (Drawing) e.Graphics.DrawLines(pen, points);
                else e.Graphics.DrawPolygon(pen, points);
            }
        }

        // Start selecting an area.
        private void picOverlay_MouseDown(object sender, MouseEventArgs e)
        {
            if (picOverlay.Image == null) return;

            Drawing = true;

            ScaledSelectedArea = new List<PointF>();
            ScaledSelectedArea.Add(e.Location);

            SelectedArea = new List<PointF>();
            SelectedArea.Add(new PointF(e.X / ImageScale, e.Y / ImageScale));

            picOverlay.Refresh();
        }

        // Continue selecting an area.
        private void picOverlay_MouseMove(object sender, MouseEventArgs e)
        {
            if (!Drawing) return;
            ScaledSelectedArea.Add(e.Location);
            SelectedArea.Add(new PointF(e.X / ImageScale, e.Y / ImageScale));
            picOverlay.Refresh();
        }

        // Finish selecting an area.
        private void picOverlay_MouseUp(object sender, MouseEventArgs e)
        {
            Drawing = false;
            picOverlay.Refresh();
            btnCopy.Enabled = (SelectedArea.Count > 2);
        }

        // Scale a bitmap.
        private Bitmap ScaleBitmap(Bitmap bm, float scale)
        {
            int scaled_width = (int)(bm.Width * scale);
            int scaled_height = (int)(bm.Height * scale);
            Bitmap result = new Bitmap(scaled_width, scaled_height);
            using (Graphics gr = Graphics.FromImage(result))
            {
                Rectangle dest_rect = new Rectangle(
                    0, 0, scaled_width, scaled_height);
                Rectangle src_rect = new Rectangle(
                    0, 0, bm.Width, bm.Height);
                gr.DrawImage(bm, dest_rect, src_rect, GraphicsUnit.Pixel);
            }
            return result;
        }

        private void mnuScale_Click(object sender, EventArgs e)
        {
            // Get the scale factor.
            ToolStripMenuItem menu_item = sender as ToolStripMenuItem;
            string scale_text = menu_item.Text.Replace("&", "").Replace("%", "");
            ImageScale = float.Parse(scale_text) / 100f;
            ShowScaledImages();

            // Display the new scale.
            mnuScale.Text = "Scale (" + menu_item.Text.Replace("&", "") + ")";

            // Check the selected menu item.
            foreach (ToolStripMenuItem item in mnuScale.DropDownItems)
            {
                item.Checked = (item == menu_item);
            }

            // Scale the selected area.
            ScaleSelectionArea();
        }

        private void ShowScaledImages()
        {
            if (OverlayImage != null)
            {
                ScaledOverlayImage = ScaleBitmap(OverlayImage, ImageScale);
                picOverlay.Image = ScaledOverlayImage;
                picOverlay.Visible = true;
            }

            if (MainImage != null)
            {
                ScaledMainImage = ScaleBitmap(MainImage, ImageScale);
                picMain.Image = ScaledMainImage;
                picMain.Visible = true;
            }

            // Enable the Save As menu item.
            mnuFileSaveAs.Enabled = (
                (OverlayImage != null) &&
                (MainImage != null));
        }

        // Scale the selection area's points and save them in the ScaledSelectedArea list.
        private void ScaleSelectionArea()
        {
            ScaledSelectedArea = new List<PointF>();
            foreach (PointF point in SelectedArea)
            {
                ScaledSelectedArea.Add(new PointF(point.X * ImageScale, point.Y * ImageScale));
            }
        }

        // Copy the selected area from the overlay
        // image onto the background image.
        private void btnCopy_Click(object sender, EventArgs e)
        {
            if (SelectedArea.Count < 3) return;
            btnCopy.Enabled = false;
            mnuFileReset.Enabled = true;

            using (Graphics gr = Graphics.FromImage(MainImage))
            {
                using (TextureBrush brush = new TextureBrush(OverlayImage))
                {
                    gr.FillPolygon(brush, SelectedArea.ToArray());
                }
            }

            ShowScaledImages();
        }

        private void mnuFileReset_Click(object sender, EventArgs e)
        {
            MainImage = new Bitmap(OriginalMainImage);
            mnuFileReset.Enabled = false;
            ShowScaledImages();
        }
    }
}
